//	bfe2 - bochs connection
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
FILE *writepipe, *readpipe;
int bochs_running, bochs_offline, bochs_restarting, bochs_pid;

// local
gint sigio_tag;

//////////////////////////////////////////////////////////////////////////


int popen2( char *command, FILE **in, FILE **err ){
	int ifd[2], efd [2];

	if(in != NULL) pipe(ifd);
	if(err != NULL) pipe(efd);

	bochs_pid = fork();
	if(bochs_pid == -1) return(0);
	if(bochs_pid == 0){
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = command;
		argv[3] = 0;
		if(in != NULL) dup2(ifd [0], STDIN_FILENO);
		if(err != NULL) dup2(efd [1], STDERR_FILENO);
		freopen("/dev/null", "a", stdout);
		execv("/bin/sh", argv);
		exit(EXIT_FAILURE);
	} else {
		if(err != NULL) *err = fdopen(efd [0], "r");
		if(in != NULL) *in = fdopen(ifd [1], "w");
        }
	return(1);
}


void sigchld_handler( int x ){
//	bochsShutdown();
bfe_quit();
}


void sigio_handler( gpointer data, gint source, GdkInputCondition condition ){
	instr_stop(1);
}


//////////////////////////////////////////////////////////////////////////


void hook_sigio( ){
	sigio_tag = gdk_input_add(fileno(readpipe), GDK_INPUT_READ, sigio_handler, NULL);
}
void unhook_sigio( ){
	gdk_input_remove(sigio_tag);
}


void bochsInit( ){
	bochs_running = 0;
	bochs_offline = 1;
	bochs_restarting = 0;
}


void bochsShutdown( ){
	signal(SIGCHLD, SIG_IGN);

	if(bochs_running){
		unhook_sigio();
		if(!bochs_offline) kill(bochs_pid, SIGINT);
		bochs_running = 0;
	}

	if(!bochs_offline){
		historyWrite("Terminating Bochs");
		fprintf(writepipe, "q\n");
		fclose(readpipe);
		fclose(writepipe);
	}

	set_ctrl_online(FALSE);

	bochs_offline = 1;
}


void bochsStart( ){
	#ifdef __DEBUG__
	 g_print("BFE DEBUG: starting bochs... ");
	#endif

	signal(SIGCHLD, sigchld_handler);
	if(!popen2(BOCHS_PATH, &writepipe, &readpipe)){
		#ifdef __DEBUG__
		 g_print("FAILED\n");
		#endif
		historyWrite("BOCHS FAILED TO START");
		return;			// failed to start bochs
	}
	setbuf(readpipe, NULL);		// disable line buffering
	setbuf(writepipe, NULL);
	bochs_offline = 0;

	set_ctrl_online(TRUE);

	historyWrite("Starting Bochs");

	prompt_read();

  // TODO: read symbols

//	fprintf(writepipe, "load-symbols \"kernel.sym\" 0x5000\n");
//	prompt_read();
//	fprintf(writepipe, "load-symbols \"memory.sym\"\n");
//	prompt_read();

	fprintf(writepipe, "s\n");

	update_bfe();

	#ifdef __DEBUG__
	 g_print("OK\n");
	#endif
}


void bochsRestart( ){         
	bochsShutdown();
	sleep(1);
	bochsStart();
	bochs_restarting = 1;

//-	fl_set_input_color(i_instruction_text, FL_BLACK, FL_BLACK);
//-	fl_addto_browser(i_list, "\t\t\t\t\tBOCHS ONLINE...");
//-	fl_activate_object(go_button);
//-	fl_activate_object(stop_button);
//-	fl_activate_object(step_button);
//-	fl_activate_object(next_button);
//-	fl_activate_object(refreshall_button);
//-	//fl_activate_object(lock_button);
//-	fl_activate_object(folder);

//-	prompt_read();
	fprintf(writepipe, "s\n");
//-	update_xforms();

//-	update_breakpoints(0);

	bochs_restarting = 0;
}


int prompt_read( ){
	char c;

	if(bochs_offline) return(1);

	#ifdef __DEBUG__
	 g_print("BFE DEBUG: reading prompt... ");
	#endif

	for(c=fgetc(readpipe); c != '<' && c != EOF; c=fgetc(readpipe));
	if(c != '<') return(1);
	if(fscanf(readpipe, "bochs:%u>", &p_step) != 1) return(1);

	#ifdef __DEBUG__
	 g_print("OK\n");
	#endif

	return(0);
}
