//	bfe2 - execution breakpoints
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
s_breakpoint breakpoints[MAX_BREAKPOINTS];

// local
GtkWidget *bp_add_button, *bp_remove_button, *bp_removeall_button, *bp_address_text;
GtkCList *bp_list;
int selected_bp, bp_type;

static gchar *column_names[] = { "", "", "" };

//////////////////////////////////////////////////////////////////////////


int read_breakpoints( ){
	int x;
	int inputChar;
	char scanChar;

	if(bochs_offline) return(0);

	// remove the "Num Type Disp Enb Address"
	inputChar = fgetc(readpipe);
	for(; !isdigit(inputChar) && (inputChar != '<') && (inputChar != EOF); inputChar = fgetc(readpipe) );
	if(inputChar == EOF) return(0);
	if(ungetc(inputChar, readpipe) == EOF) return(0);

	for(x = 0; x < MAX_BREAKPOINTS; x++){
		// remove what?
		inputChar = fgetc(readpipe);
		for(; isspace(inputChar) && (inputChar != EOF); inputChar = fgetc(readpipe) );
		if(inputChar == EOF) return(x);
		if((ungetc(inputChar, readpipe) == EOF) || !isdigit(inputChar)) return(x);

		if(fscanf(readpipe, "%i %c%*s ", &breakpoints[x].number, &scanChar) != 2) return(x);
		switch(scanChar){
		case 'p':
			breakpoints[x].type = BP_PHYSICAL;
			break;
		case 'l':
			breakpoints[x].type = BP_LINEAR;
			break;
		case 'v':
			breakpoints[x].type = BP_VIRTUAL;
			break;
		default:
			return(x);
		}
		if(fscanf(readpipe, "%*s %c ", &scanChar) != 1) return(x);
		breakpoints[x].enabled = (scanChar == 'y');
		if(fscanf(readpipe, "%x", & breakpoints[x].segment) != 1) return(x);
		if(fscanf(readpipe, ":%x", &breakpoints[x].offset) < 1){
			breakpoints[x].offset  = breakpoints[x].segment;
			breakpoints[x].segment = 0;
		}
	}
	return(x);
}



void set_breakpoint_type( GtkWidget *widget, gpointer data ){
	switch((int)data){
	case BP_PHYSICAL:
		bp_type = BP_PHYSICAL;
		break;
	case BP_LINEAR:
		bp_type = BP_LINEAR;
		break;
	case BP_VIRTUAL:
		bp_type = BP_VIRTUAL;
		break;
	}
}


void add_breakpoint( GtkWidget *widget, gpointer data ){
	const char *address;

	address = gtk_entry_get_text(GTK_ENTRY(bp_address_text));

	switch(bp_type){
	case BP_PHYSICAL:
		fprintf(writepipe, "pb %s\n", address);
		break;
	case BP_LINEAR:
		fprintf(writepipe, "lb %s\n", address);
		break;
	case BP_VIRTUAL:
		fprintf(writepipe, "vb %s\n", address);
		break;
	}
	prompt_read();
	if(!bochs_restarting) breakpointsUpdate(0);
	gtk_entry_set_text(GTK_ENTRY(bp_address_text), "");
}


void remove_breakpoint( GtkWidget *widget, gpointer data ){
	if(selected_bp == -1) return;

	fprintf(writepipe, "d %d\n", breakpoints[selected_bp].number);
	prompt_read();
	breakpointsUpdate(0);
}


void removeall_breakpoints( GtkWidget *widget, gpointer data ){
	int x;

	for(x = 0; x < MAX_BREAKPOINTS; x++){
		if(breakpoints[0].enabled){
			fprintf(writepipe, "d %d\n", breakpoints[0].number);
			prompt_read();
			breakpointsUpdate(0);
		} else break;
	}
}


void bp_selected( GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data ){
	selected_bp = row;
}


void bp_unselected( GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data ){
	selected_bp = -1;
}

//////////////////////////////////////////////////////////////////////////


void breakpointsInit( GtkWidget *vbox ){
	GtkWidget *hbox, *vbox2, *scrolled_window, *button;

	hbox = new_hbox(vbox, TRUE);

	// create scrolled window
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	// create list
	bp_list = (GtkCList *)gtk_clist_new_with_titles(3, column_names);
	gtk_clist_set_column_auto_resize(GTK_CLIST(bp_list), 0, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(bp_list), 1, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(bp_list), 2, TRUE);
	gtk_clist_set_selection_mode(bp_list, GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(bp_list));
	gtk_widget_show(GTK_WIDGET(bp_list));

	gtk_clist_column_titles_hide(bp_list);

	// setup selection handler
	gtk_clist_set_selection_mode(bp_list, GTK_SELECTION_SINGLE);
	gtk_signal_connect(GTK_OBJECT(bp_list), "select_row", GTK_SIGNAL_FUNC(bp_selected), NULL);
	gtk_signal_connect(GTK_OBJECT(bp_list), "unselect_row", GTK_SIGNAL_FUNC(bp_unselected), NULL);

	vbox2 = new_vbox(hbox, FALSE);
	hbox = new_hbox(vbox2, FALSE);

	// add button
	bp_add_button = gtk_button_new_with_label(" Add ");
	gtk_signal_connect(GTK_OBJECT(bp_add_button), "clicked", GTK_SIGNAL_FUNC(add_breakpoint), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), bp_add_button, TRUE, TRUE, 0);
	gtk_widget_show(bp_add_button);

	// remove button
	bp_remove_button = gtk_button_new_with_label(" Remove ");
	gtk_signal_connect(GTK_OBJECT(bp_remove_button), "clicked", GTK_SIGNAL_FUNC(remove_breakpoint), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), bp_remove_button, FALSE, FALSE, 0);
	gtk_widget_show(bp_remove_button);

	// remove all button
	bp_removeall_button = gtk_button_new_with_label(" Remove All ");
	gtk_signal_connect(GTK_OBJECT(bp_removeall_button), "clicked", GTK_SIGNAL_FUNC(removeall_breakpoints), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), bp_removeall_button, FALSE, FALSE, 0);
	gtk_widget_show(bp_removeall_button);

	hbox = new_hbox(vbox2, FALSE);

	// physical radio button
	button = gtk_radio_button_new_with_label(NULL, "Physical");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(set_breakpoint_type), (gpointer)BP_PHYSICAL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	// linear radio button
	button = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(button)), "Linear");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(set_breakpoint_type), (gpointer)BP_LINEAR);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	// virtual radio button
	button = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(button)), "Virtual");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(set_breakpoint_type), (gpointer)BP_VIRTUAL);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	// address text field
	bp_address_text = gtk_entry_new();
	gtk_widget_set_usize(bp_address_text, 145, 0);
	gtk_signal_connect(GTK_OBJECT(bp_address_text), "activate", GTK_SIGNAL_FUNC(add_breakpoint), NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), bp_address_text, FALSE, FALSE, 0);
	gtk_widget_show(bp_address_text);

	selected_bp = 0;
	bp_type = BP_PHYSICAL;
}


void breakpointsUpdate( int redraw ){
	int x, length;
	char address[20], *row[3];
	const char *p_type = "Physical", *l_type = "Linear", *v_type = "Virtual";

	if(bochs_offline) return;

	if(!bochs_restarting && !redraw){
		for(x = 0; x < MAX_BREAKPOINTS; x++) breakpoints[x].enabled = 0;
		fprintf(writepipe, "info break\n");
		read_breakpoints();
		prompt_read();
	}

	gtk_clist_clear(bp_list);

	row[1] = address;
	row[2] = NULL;
	length = 0;

	for(x = 0; (x < MAX_BREAKPOINTS) && breakpoints[x].enabled; x++){
		switch(breakpoints[x].type){
		case BP_PHYSICAL:
			row[0] = (char *)p_type;
			snprintf(address, 20, "0x%.8X", breakpoints[x].offset);
			break;
		case BP_LINEAR:
			row[0] = (char *)l_type;
			snprintf(address, 20, "0x%.8X", breakpoints[x].offset);
			break;
		case BP_VIRTUAL:
			row[0] = (char *)v_type;
			snprintf(address, 20, "0x%.4X:0x%.8X", breakpoints[x].segment, breakpoints[x].offset);
			break;
		}
		gtk_clist_append(bp_list, row);
		length++;

		if(bochs_restarting){
//			bp_type = breakpoints[x].type;
//			gtk_entry_set_text(GTK_ENTRY(bp_address_text), address);
//			add_breakpoint(NULL, 0);

  // TODO: need to restore bp_type or else the radio buttons won't match

		}
	}
	if(breakpoints[0].enabled && !redraw) gtk_clist_select_row(bp_list, length-1, 0);
}


void breakpointsEnablePage( gboolean v ){
	gtk_widget_set_sensitive(bp_add_button, v);
	gtk_widget_set_sensitive(bp_remove_button, v);
	gtk_widget_set_sensitive(bp_removeall_button, v);
	gtk_widget_set_sensitive(bp_address_text, v);
}


void breakpointsSelect( int bp ){
	gtk_clist_set_text(bp_list, bp, 2, "<--");
	gtk_clist_select_row(bp_list, bp, 0);
}
