//	bfe2 - globals
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <gtk/gtk.h>


////////////////////////////////////////////
// path to the bochs binary
#define BOCHS_PATH "bochs"
// path to bfe runtime config file
#define CONFIG_PATH ".bferc"
// path to the history log gile
#define HISTORY_PATH "__bfe.log__"
// uncomment to disable the quit confirmation
#define __NO_QUIT_CONFIRM__
// uncomment to enable debugging output to console
// #define __DEBUG__
////////////////////////////////////////////


#define MAX_BREAKPOINTS 30
#define MAX_WATCHPOINTS 30

#define DEF_HISLEN 50
#define DEF_STACKABOVE 8
#define DEF_STACKBELOW 4
#define DEF_STEPLOCK 2
#define DEF_NEXTLOCK 2

#ifndef uint32
	typedef unsigned int uint32;
#endif
#ifndef int32
	typedef int int32;
#endif
#ifndef uint64
	typedef unsigned long long uint64;
#endif
#ifndef int64
	typedef long long int64;
#endif


//
// main.c
//

enum { PAGE_BP, PAGE_WP, PAGE_MEM, PAGE_STRUC, PAGE_CFG, PAGE_HIST, MAX_PAGES };

extern GtkWidget *address_text, *offset_text, *instruction_text;
extern GtkWidget *go_button, *stop_button, *step_button, *next_button, *refreshall_button;
extern int show_state;
extern int next_active, tracing;


//
// misc.c
//

#define STRING_LENGTH 128


//
// bochs.c
//

extern FILE *writepipe, *readpipe;
extern int bochs_running, bochs_offline, bochs_restarting, bochs_pid;


//
// control.c
//

extern uint32 steplock_time, nextlock_time;


//
// state.c
//

typedef struct s_segment {
	uint32 segment;
	uint32 descriptor[2];
	uint32 valid;
} s_segment;
typedef struct s_table {
	uint32 base;
	uint32 limit;
} s_table;
typedef struct s_cpu {
	uint32 eax;
	uint32 ebx;
	uint32 ecx;
	uint32 edx;
	uint32 ebp;
	uint32 esi;
	uint32 edi;
	uint32 esp;
	uint32 eflags;
	uint32 eip;
	struct s_segment cs;
	struct s_segment ss;
	struct s_segment ds;
	struct s_segment es;
	struct s_segment fs;
	struct s_segment gs;
	struct s_segment ldtr;
	struct s_segment tr;
	struct s_table gdtr;
	struct s_table idtr;
	uint32 dr0;
	uint32 dr1;
	uint32 dr2;
	uint32 dr3;
	uint32 dr4;
	uint32 dr5;
	uint32 dr6;
	uint32 dr7;
	uint32 tr3;
	uint32 tr4;
	uint32 tr5;
	uint32 tr6;
	uint32 tr7;
	uint32 cr0;
	uint32 cr1;
	uint32 cr2;
	uint32 cr3;
	uint32 cr4;
	uint32 inhibit;
} s_cpu;

extern struct s_cpu cpu;
extern uint32 i_virtual_segment, i_virtual_offset, i_physical_address, i_size;
extern char i_description[STRING_LENGTH];
extern int p_step;
extern uint32 stack_above, stack_below;


//
// breakpoints.c
//

enum { BP_PHYSICAL, BP_LINEAR, BP_VIRTUAL };
typedef struct s_breakpoint {
	int number;
	int type;
	int enabled;
	uint32 segment;
	uint32 offset;
} s_breakpoint;

extern struct s_breakpoint breakpoints[MAX_BREAKPOINTS];


//
// watchpoints.c
//

enum { WP_READ, WP_WRITE };
typedef struct s_watchpoint {
	int type;
	int enabled;
	uint32 offset;
} s_watchpoint;

extern struct s_watchpoint watchpoints[MAX_WATCHPOINTS];


//
// memory.c
//

typedef struct s_memwin {
	GtkWidget *window;
	GtkWidget *list, *close_button, *refresh_button;
	uint32 address;
	uint32 units;
	uint32 physical;
	char unit_size;
	char unit_format;
	struct s_memwin *prev;
	struct s_memwin *next;
} s_memwin;

extern struct s_memwin *mem_windows;
extern char mem_type, mem_size, mem_physical;


//
// structures.c
//

#define NEWLINE 10
#define SPACE 32
#define TAB 9
#define WORD_LEN 99

typedef struct s_value {
	char name[WORD_LEN];
	size_t size;
} s_value;
typedef struct s_mem {
	GtkWidget *window;
	GtkWidget *start_text, *total_text;
	GtkWidget *close_button, *refresh_button, *table;
	char name[WORD_LEN];
	uint32 base;
	size_t element_count;
	size_t value_count;
	struct s_value *values;
} s_mem;

extern struct s_mem *memlist;
extern int nr_strucs;


//
// config.c
//


//
// history.c
//

extern uint32 h_maxlen;
