//	bfe2 - configuration
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global

// local
GtkWidget *hislen_text, *stackabove_text, *stackbelow_text, *steplock_text, *nextlock_text;
GtkWidget *trace_button;

//////////////////////////////////////////////////////////////////////////


void word_read( FILE * const file, register char * str, const size_t len ){
	register int inputChar;
	register char * end;

	do {
	inputChar = fgetc (file);
	} while(inputChar != EOF && isspace(inputChar) != 0);
	for(end = str + len - 1;
	    str < end && inputChar != EOF && isspace(inputChar) == 0;
	    ++str, inputChar = fgetc(file))
		*str = inputChar;
	*str = '\0';
	while(inputChar != EOF && isspace(inputChar) == 0)
		inputChar = fgetc(file);
	(void) ungetc(inputChar, file);
}


void config_load( FILE *file ){
	register int inputChar;
	char typeChar;
	s_mem *tmp, input;
	s_value *current, *end;

	memlist = NULL;
	nr_strucs = 0;

	for(;;){
		do {
			inputChar = fgetc(file);
		} while(inputChar != EOF && inputChar != '{');
		if(inputChar == EOF)
			return;
		if(fscanf(file, " %c", &typeChar) != 1)
			return;
		inputChar = typeChar;
		if(typeChar == 'H' || typeChar == 'h'){
			while(inputChar != EOF && isspace(inputChar) == 0)
				inputChar = fgetc(file);
			if(fscanf(file, " %i", &h_maxlen) != 1)
				return;
			continue;
		}

		if(typeChar == 'S' || typeChar == 's'){
			inputChar = fgetc(file);
			if(inputChar == 'T' || inputChar == 't'){
				inputChar = fgetc(file);
				if(inputChar == 'A' || inputChar == 'a'){
					while(inputChar != EOF && isspace(inputChar) == 0)
						inputChar = fgetc(file);
					if(fscanf(file, " %i %i", &stack_above, &stack_below) != 2)
						return;
					continue;
				}
			}
			input.element_count = 1;
		}
		while(inputChar != EOF && isspace(inputChar) == 0)
			inputChar = fgetc(file);
		if(inputChar == EOF)
			return;
		input.name [0] = inputChar;
		word_read(file, input.name + 1, WORD_LEN - 1);
		if(fscanf(file, " %i", &input.base) != 1)
			return;
		if((typeChar == 'L' || typeChar == 'l') && fscanf(file, " %i", &input.element_count) != 1)
			return;
		if(fscanf(file, " %i", &input.value_count) != 1)
			return;
		if(input.value_count > 0)
			input.values = (s_value *)malloc(sizeof(s_value) * input.value_count);
		if(input.values == NULL)
			return;

		for(current = input.values, end = current + input.value_count; current < end; ++current){
			word_read(file, current->name, WORD_LEN);
			if(fscanf(file, " %i", &current->size) != 1){
				free(input.values);
				return;
			}
		}

		do {
			inputChar = fgetc(file);
		} while(inputChar != EOF && inputChar != '}');
		tmp = (s_mem *)realloc(memlist, sizeof(s_mem) * (++nr_strucs));
		if(tmp == NULL){
			nr_strucs--;
			return;
		}
		memlist = tmp;
		memlist [nr_strucs - 1] = input;
	}
}


void toggle_tracing( GtkWidget *widget, gpointer data ){
	if(tracing){
		tracing = 0;
		historyWrite("--- TRACE DISABLED ---");
		fprintf(writepipe, "trace-off\n");
		prompt_read();
	} else {
		tracing = 1;
		historyWrite("--- TRACE ENABLED ---");
		fprintf(writepipe, "trace-on\n");
		prompt_read();
	}
}


void update_text( GtkWidget *text, int value ){
	char temp[16];

	snprintf(temp, 16, "%d", value);
	gtk_entry_set_text(GTK_ENTRY(text), temp);
}
void set_cfg_hislen( GtkWidget *widget, gpointer data ){
	h_maxlen = atoi(gtk_entry_get_text(GTK_ENTRY(hislen_text)));
	if(h_maxlen < 1) h_maxlen = 1;
	historyClip();
	update_text(widget, h_maxlen);
}
void set_cfg_stackabove( GtkWidget *widget, gpointer data ){
	stack_above = atoi(gtk_entry_get_text(GTK_ENTRY(stackabove_text)));
	if(stack_above < 0) stack_above = 0;
	stateUpdateStack();
	update_text(widget, stack_above);
}
void set_cfg_stackbelow( GtkWidget *widget, gpointer data ){
	stack_below = atoi(gtk_entry_get_text(GTK_ENTRY(stackbelow_text)));
	if(stack_below < 0) stack_below = 0;
	stateUpdateStack();
	update_text(widget, stack_below);
}
void set_cfg_steplock( GtkWidget *widget, gpointer data ){
	steplock_time = atoi(gtk_entry_get_text(GTK_ENTRY(steplock_text)));
	if(steplock_time < 1) steplock_time = 1;
	update_text(widget, steplock_time);
}
void set_cfg_nextlock( GtkWidget *widget, gpointer data ){
	nextlock_time = atoi(gtk_entry_get_text(GTK_ENTRY(nextlock_text)));
	if(nextlock_time < 1) nextlock_time = 1;
	update_text(widget, nextlock_time);
}


static GtkWidget *new_text_entry( GtkWidget *box, char *name, int maxlen, void (*callback)(), int width ){
	GtkWidget *hbox, *text, *label;

	hbox = new_hbox(box, FALSE);

	text = gtk_entry_new_with_max_length(maxlen);
	gtk_entry_set_editable(GTK_ENTRY(text), TRUE);
	gtk_widget_set_usize(text, width, 20);
	gtk_signal_connect(GTK_OBJECT(text), "activate", GTK_SIGNAL_FUNC(callback), NULL);
//	gtk_signal_connect(GTK_OBJECT(text), "changed", GTK_SIGNAL_FUNC(callback), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), text, FALSE, FALSE, 0);
	gtk_widget_show(text);

	label = gtk_label_new(name);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	return(text);
}


//////////////////////////////////////////////////////////////////////////


void configInit( GtkWidget *vbox ){
	GtkWidget *hbox, *vbox2, *button;
	FILE *file;

	hbox = new_hbox(vbox, FALSE);

	vbox2 = new_vbox(hbox, FALSE);

	hislen_text = new_text_entry(vbox2, "Instruction History Length", 5, set_cfg_hislen, 75);
	stackabove_text = new_text_entry(vbox2, "Stack values above ESP", 4, set_cfg_stackabove, 75);
	stackbelow_text = new_text_entry(vbox2, "Stack values below ESP", 4, set_cfg_stackbelow, 75);
	steplock_text = new_text_entry(vbox2, "Step-Lock Time (seconds)", 3, set_cfg_steplock, 75);
	nextlock_text = new_text_entry(vbox2, "Next-Lock Time (seconds)", 3, set_cfg_nextlock, 75);

	/////////////////////////////////////////////

	vbox2 = new_vbox(hbox, FALSE);

	trace_button = gtk_check_button_new_with_label("Enable instruction tracing");
	gtk_signal_connect(GTK_OBJECT(trace_button), "clicked", GTK_SIGNAL_FUNC(toggle_tracing), NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), trace_button, FALSE, FALSE, 0);
	gtk_widget_show(trace_button);

	button = gtk_button_new_with_label("Clear History");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(historyClear), NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	/////////////////////////////////////////////

	// defaults settings
	h_maxlen = DEF_HISLEN;
	stack_above = DEF_STACKABOVE;
	stack_below = DEF_STACKBELOW;
	steplock_time = DEF_STEPLOCK;
	nextlock_time = DEF_NEXTLOCK;
	tracing = 0;

	if((file = fopen(CONFIG_PATH, "r")) == NULL){
		g_print("BFE: Config file not found, using defaults\n");
	} else {
		config_load(file);
	}
	configUpdate();
}


void configUpdate( ){
	char temp[200];

	snprintf(temp, 200, "%d", h_maxlen);
	gtk_entry_set_text(GTK_ENTRY(hislen_text), temp);
	snprintf(temp, 200, "%d", stack_above);
	gtk_entry_set_text(GTK_ENTRY(stackabove_text), temp);
	snprintf(temp, 200, "%d", stack_below);
	gtk_entry_set_text(GTK_ENTRY(stackbelow_text), temp);
	snprintf(temp, 200, "%d", steplock_time);
	gtk_entry_set_text(GTK_ENTRY(steplock_text), temp);
	snprintf(temp, 200, "%d", nextlock_time);
	gtk_entry_set_text(GTK_ENTRY(nextlock_text), temp);
}


void configEnablePage( gboolean v ){
	gtk_widget_set_sensitive(trace_button, v);
}
