//	bfe2 - instruction history file
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
uint32 h_maxlen;

// local
GtkCList *h_list;
FILE *h_logfile;
uint32 h_last_address, h_repeat, h_length;

static gchar *column_names[] = { "", "", "" };

//////////////////////////////////////////////////////////////////////////


void historyOpen( ){
	h_last_address = 0;
	h_repeat = 0;
	h_length = 0;
	if((h_logfile = fopen(HISTORY_PATH, "w")) == NULL)
		g_print("BFE: Unable to access history log file, logging disabled.\n");
}


void historyGotoEnd( ){
	gtk_clist_moveto(h_list, h_length-1, 0, 1.0, 0.0);
}


//////////////////////////////////////////////////////////////////////////


void historyInit( GtkWidget *vbox ){
	GtkWidget *scrolled_window;

	historyOpen();

	// create scrolled window
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	// create list
	h_list = (GtkCList *)gtk_clist_new_with_titles(3, column_names);
	gtk_clist_set_column_auto_resize(GTK_CLIST(h_list), 0, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(h_list), 1, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(h_list), 2, TRUE);
	gtk_clist_set_selection_mode(h_list, GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(h_list));
	gtk_widget_show(GTK_WIDGET(h_list));

	gtk_clist_column_titles_hide(h_list);
}


void historyClose( ){
	if(h_logfile != NULL) fclose(h_logfile);
}


void historyClear( GtkWidget *widget, gpointer data ){
	gtk_clist_clear(h_list);

	historyClose();
	historyOpen();
}


void historyUpdate( ){
	char address[16], offset[20], *row[3];

	if(h_last_address == i_physical_address){
		h_repeat++;
	} else {
		h_last_address = i_physical_address;

		historyRepeat();

		snprintf(address, 16, "0x%.8X", i_physical_address);
		snprintf(offset, 20, "0x%.4X:0x%.8X", i_virtual_segment, i_virtual_offset);
		row[0] = address;
		row[1] = offset;
		row[2] = i_description;
		gtk_clist_append(h_list, row);
		h_length++;

		if(h_logfile != NULL) fprintf(h_logfile, "%s  %s  %s\n", address, offset, i_description);

		historyClip();

		gtk_clist_select_row(h_list, h_length-1, 0);

		historyGotoEnd();
	}
}


void historyRepeat( ){
	char temp[64];

	if(h_repeat){
		snprintf(temp, 64, "repeated %d times", h_repeat);
		historyWrite(temp);
		h_repeat = 0;
	}
}


void historyWrite( char *message ){
	char *row[3];
	const char *c = "";

	row[0] = row[1] = (char *)c;
	row[2] = message;
	gtk_clist_append(h_list, row);
	h_length++;

	if(h_logfile != NULL)
		fprintf(h_logfile, ": %s\n", message);

	historyClip();

	historyGotoEnd();
}


void historyClip( ){
	while(h_maxlen < h_length){
		gtk_clist_remove(h_list, 0);
		h_length--;
	}
}
