//      bfe2 - main
//      Copyright (c) 1999-2003 Brand Huntsman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
GtkWidget *address_text, *offset_text, *instruction_text;
GtkWidget *go_button, *stop_button, *step_button, *next_button, *refreshall_button;
int show_state;
int next_active, tracing;

// local
GtkWidget *state_hbox, *quit_window, *notebook, *bochs_button, *state_button;
static guint id;

//////////////////////////////////////////////////////////////////////////


gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data ){
        return(FALSE);
}
void destroy_window( GtkWidget *widget, gpointer data ){
	bfe_quit();
}


void toggle_state( GtkWidget *widget, gpointer data ){
	if(show_state){
		gtk_widget_hide(state_hbox);
		gtk_object_set(GTK_OBJECT(state_button), "label", " Show State ", NULL);
		show_state = 0;
	} else {
		gtk_widget_show(state_hbox);
		gtk_object_set(GTK_OBJECT(state_button), "label", " Hide State ", NULL);
		stateUpdate();
		show_state = 1;
	}
}


void toggle_bochs( GtkWidget *widget, gpointer data ){
	if(bochs_offline){
		bochsStart();
		gtk_object_set(GTK_OBJECT(bochs_button), "label", " Kill Bochs ", NULL);
		if(bochs_offline){
			gtk_signal_disconnect(GTK_OBJECT(widget), id);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), FALSE);
			id = gtk_signal_connect(GTK_OBJECT(widget), "clicked", GTK_SIGNAL_FUNC(toggle_bochs), NULL);
			gtk_object_set(GTK_OBJECT(bochs_button), "label", " Start Bochs ", NULL);
		}
	} else {
		bochsShutdown();
		gtk_object_set(GTK_OBJECT(bochs_button), "label", " Start Bochs ", NULL);
	}
}


void destroy_quit_window( GtkWidget *widget, gpointer data ){
	gtk_widget_destroy(quit_window);
}
void confirm_quit( ){
	#ifdef __NO_QUIT_CONFIRM__
		bfe_quit();
	#else
		GtkWidget *vbox, *hbox, *label, *button;

		// create quit confirmation window
		quit_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_signal_connect(GTK_OBJECT(quit_window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
		gtk_signal_connect(GTK_OBJECT(quit_window), "destroy", GTK_SIGNAL_FUNC(destroy_quit_window), NULL);
		gtk_container_set_border_width(GTK_CONTAINER(quit_window), 5);
		gtk_widget_set_usize(GTK_WIDGET(quit_window), 300, 100);
		gtk_window_set_title(GTK_WINDOW(quit_window), "BFE - Quit?");
		gtk_widget_show(quit_window);

		// create window vbox
		vbox = gtk_vbox_new(FALSE, 5);
		gtk_container_add(GTK_CONTAINER(quit_window), vbox);
		gtk_widget_show(vbox);

		// confirmation label
		label = gtk_label_new("Do you really want to quit?");
		gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);

		hbox = new_hbox(vbox, FALSE);

		// yes button
		button = gtk_button_new_with_label(" Yes ");
		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(destroy_window), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		// no button
		button = gtk_button_new_with_label(" No ");
		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(destroy_quit_window), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
		gtk_widget_show(button);
	#endif
}


GtkWidget *add_scrolled_page( const char *name ){
	GtkWidget *scrolled_window, *vbox;

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, gtk_label_new(name));
	gtk_widget_show(vbox);

	// create scrolled window
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	vbox = gtk_vbox_new(FALSE, 25);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), vbox);
	gtk_widget_show(vbox);

	return(vbox);
}


GtkWidget *add_page( const char *name ){
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, gtk_label_new(name));
	gtk_widget_show(vbox);

	return(vbox);
}


void bochs_go( GtkWidget *widget, gpointer data ){
	instr_go();
}
void bochs_stop( GtkWidget *widget, gpointer data ){
	instr_stop(0);
}
void bochs_step( GtkWidget *widget, gpointer data ){
	instr_step();
}
void bochs_next( GtkWidget *widget, gpointer data ){
	instr_next();
}


void refreshall_windows( GtkWidget *widget, gpointer data ){
	g_print("BFE: refresh all windows (FINISH ME)\n");
/*
	for(x=0; x < nr_strucs; x++) update_structure(x);
	m = mem_windows;
	while(m != NULL){
		update_memory((long)m);
		m = m->next;
	}
*/
}


//////////////////////////////////////////////////////////////////////////


void bfe_quit( ){
	historyRepeat();
	historyClose();
	bochsShutdown();
	gtk_main_quit();
}


int main( int argc, char *argv[] ){
	GtkWidget *window, *window_vbox, *hbox, *button;
	char temp[200];

	show_state = 0;
	bochsInit();

	next_active = 0;

	gtk_init(&argc, &argv);
	snprintf(temp, 200, "%s/.gtkrc", getenv("HOME"));
	gtk_rc_parse(temp);

	// create window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy_window), NULL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	gtk_widget_set_usize(GTK_WIDGET(window), 630, 500);
	gtk_window_set_title(GTK_WINDOW(window), "BFE -- http://www.qzx.com/bfe");
	gtk_widget_show(window);

	// create window vbox
	window_vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(window), window_vbox);
	gtk_widget_show(window_vbox);

	/////////////////////////////////////////////

	hbox = new_hbox(window_vbox, FALSE);

	// address text field
	address_text = gtk_entry_new();
	gtk_widget_set_usize(address_text, 90, 0);
	gtk_entry_set_editable(GTK_ENTRY(address_text), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), address_text, FALSE, FALSE, 0);
	gtk_widget_show(address_text);

	// offset text field
	offset_text = gtk_entry_new();
	gtk_widget_set_usize(offset_text, 145, 0);
	gtk_entry_set_editable(GTK_ENTRY(offset_text), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), offset_text, FALSE, FALSE, 0);
	gtk_widget_show(offset_text);

	// instruction text field
	instruction_text = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(instruction_text), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), instruction_text, TRUE, TRUE, 0);
	gtk_widget_show(instruction_text);

	// go button
	go_button = gtk_button_new_with_label(" Go ");
	gtk_signal_connect(GTK_OBJECT(go_button), "clicked", GTK_SIGNAL_FUNC(bochs_go), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), go_button, FALSE, FALSE, 0);
	gtk_widget_show(go_button);

	// stop button
	stop_button = gtk_button_new_with_label(" Stop ");
	gtk_signal_connect(GTK_OBJECT(stop_button), "clicked", GTK_SIGNAL_FUNC(bochs_stop), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), stop_button, FALSE, FALSE, 0);
	gtk_widget_show(stop_button);

	// step button
	step_button = gtk_button_new_with_label(" Step ");
	gtk_signal_connect(GTK_OBJECT(step_button), "clicked", GTK_SIGNAL_FUNC(bochs_step), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), step_button, FALSE, FALSE, 0);
	gtk_widget_show(step_button);

	// next button
	next_button = gtk_button_new_with_label(" Next ");
	gtk_signal_connect(GTK_OBJECT(next_button), "clicked", GTK_SIGNAL_FUNC(bochs_next), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), next_button, FALSE, FALSE, 0);
	gtk_widget_show(next_button);

	/////////////////////////////////////////////

	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
	gtk_box_pack_start(GTK_BOX(window_vbox), notebook, TRUE, TRUE, 0);
	gtk_widget_show(notebook);

	breakpointsInit(add_scrolled_page("Breakpoints"));
	watchpointsInit(add_scrolled_page("Watchpoints"));
	memoryInit(add_scrolled_page("Memory"));
	structuresInit(add_scrolled_page("Structures"));
	configInit(add_scrolled_page("Config"));
	historyInit(add_page("History"));

	/////////////////////////////////////////////

	state_hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(window_vbox), state_hbox, FALSE, FALSE, 0);

	stateInit(state_hbox);

	/////////////////////////////////////////////

	hbox = new_hbox(window_vbox, FALSE);

	// start button
	bochs_button = gtk_toggle_button_new_with_label(" Start Bochs ");
	id = gtk_signal_connect(GTK_OBJECT(bochs_button), "clicked", GTK_SIGNAL_FUNC(toggle_bochs), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), bochs_button, FALSE, FALSE, 0);
	gtk_widget_show(bochs_button);

	// refresh-all button
	refreshall_button = gtk_button_new_with_label(" Refresh All Windows ");
	gtk_signal_connect(GTK_OBJECT(refreshall_button), "clicked", GTK_SIGNAL_FUNC(refreshall_windows), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), refreshall_button, FALSE, FALSE, 0);
	gtk_widget_show(refreshall_button);

	// state button
	state_button = gtk_toggle_button_new_with_label(" Show State ");
	gtk_signal_connect(GTK_OBJECT(state_button), "clicked", GTK_SIGNAL_FUNC(toggle_state), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), state_button, TRUE, TRUE, 0);
	gtk_widget_show(state_button);

	// quit button
	button = gtk_button_new_with_label(" Quit ");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(confirm_quit), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	/////////////////////////////////////////////

	bochsShutdown();
	memoryUpdate();
	structuresUpdate();
	update_bfe();

	gtk_main();
	exit(0);
}
