//	bfe2 - miscellaneous functions
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global

// local

//////////////////////////////////////////////////////////////////////////


int instruction_read( ){
	char buffer[200];
	int n;

	if(bochs_offline) return(1);

	n = 1;
	fgets(buffer, 200, readpipe); // get "Next at t=1"
	fgets(buffer, 200, readpipe); // get instruction

	if(sscanf(&buffer[n], "%*s %x:%x: ", &i_virtual_segment, &i_virtual_offset) != 2) return(1);
	n = remove_word(buffer, n, 200); n = remove_word(buffer, n, 200);

	if(buffer[n] == '('){
		n = remove_word(buffer, n, 200); n = remove_word(buffer, n, 200);
	}
	sscanf(&buffer[n], "%*x%n: ", &i_size);
	i_size/=2;

	n = remove_word(buffer, n, 200);
	string_read(&buffer[n], i_description, '\n');
	if(i_description == NULL) return(1);
	return(0);
}


//////////////////////////////////////////////////////////////////////////


int get_physical( long segment, uint32 offset , uint32 *physical ){
	s_segment *seg;
	uint32 limit, base;

	seg = (s_segment *)segment;
	if(!seg->valid) return(0);

	limit = (seg->descriptor[0] & 0xffff) + (seg->descriptor[1] & 0xf0000);
	if((seg->descriptor[1] & 0x400000) != 0) limit = limit << 12;

	base = (seg->descriptor[0] >> 16) + ((seg->descriptor[1] & 0xff) << 16);
	base += seg->descriptor[1] & 0xff000000;

	*physical = base + offset;

	if((base + offset) > (base + limit)) return(0);
	return(1);
}


void update_bfe( ){
	char temp[50];

	if(bochs_offline) return;

	#ifdef __DEBUG__
	 g_print("BFE DEBUG: (UPDATE) prompt = %d\n", p_step); 
	#endif

	instruction_read();
	gtk_entry_set_text(GTK_ENTRY(instruction_text), i_description);
	snprintf(temp, 50, "0x%X:0x%X", i_virtual_segment, i_virtual_offset);
	gtk_entry_set_text(GTK_ENTRY(offset_text), temp);
	prompt_read();

	fprintf(writepipe, "dump_cpu\n");
	read_cpu();
	prompt_read();

	if(!get_physical((long)&cpu.cs, i_virtual_offset, &i_physical_address))
		i_physical_address = 0;
	snprintf(temp, 50, "0x%X", i_physical_address);
	gtk_entry_set_text(GTK_ENTRY(address_text), temp);

	historyUpdate();

	if(show_state) stateUpdate();
}


int string_read( char *buffer, char *string, const char delimiter ){
	register char *current, *end, c;
	register int n;

	n = 0;
	current = string;
	end = string + STRING_LENGTH;

	while(current < end){
		c = buffer[n]; n++;
		if((c == delimiter) || (c == EOF)){
			do{
				--current;
			} while((current >= string) && (isspace(*current) != 0));
			*(++current) = '\0';
			return(0);
		}
		*current = c;
		++current;
	}
	*(--end) = '\0';
	return(0);
}


int remove_word( char *buffer, int current, int max ){
	for(; current < max
		&& buffer[current] != '\0'
		&& buffer[current] != '\n'
		&& buffer[current] != '\t'
		&& buffer[current] != ' '; current++);
	while(buffer[current] == ' ' || buffer[current] == '\t') ++current;
	return(current);
}


GtkWidget *new_hbox( GtkWidget *box, gboolean pack ){
	GtkWidget *hbox;

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, pack, pack, 0);
	gtk_widget_show(hbox);

	return(hbox);
}


GtkWidget *new_vbox( GtkWidget *box, gboolean pack ){
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), vbox, pack, pack, 0);
	gtk_widget_show(vbox);

	return(vbox);
}


void new_gap( GtkWidget *box ){
	GtkWidget *sep;

	sep = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(box), sep, TRUE, TRUE, 0);
	gtk_widget_show(sep);
}


void set_ctrl_online( gboolean v ){
	if(v == FALSE){
		gtk_entry_set_text(GTK_ENTRY(address_text), "");
		gtk_entry_set_text(GTK_ENTRY(offset_text), "");
		gtk_entry_set_text(GTK_ENTRY(instruction_text), ".. o f f l i n e ..");

		gtk_widget_set_sensitive(stop_button, FALSE);
	}

	gtk_widget_set_sensitive(go_button, v);
	gtk_widget_set_sensitive(step_button, v);
	gtk_widget_set_sensitive(next_button, v);
	gtk_widget_set_sensitive(refreshall_button, v);

	breakpointsEnablePage(v);
	watchpointsEnablePage(v);
	memoryEnablePage(v);
	structuresEnablePage(v);
	configEnablePage(v);
}


void set_ctrl_running( gboolean v ){
	if(v == TRUE){
		gtk_entry_set_text(GTK_ENTRY(address_text), "");
		gtk_entry_set_text(GTK_ENTRY(offset_text), "");
		gtk_entry_set_text(GTK_ENTRY(instruction_text), ".. r u n n i n g ..");

		gtk_widget_set_sensitive(stop_button, TRUE);
		v = FALSE;
	} else {
		gtk_widget_set_sensitive(stop_button, FALSE);
		v = TRUE;
	}

	gtk_widget_set_sensitive(go_button, v);
	gtk_widget_set_sensitive(step_button, v);
	gtk_widget_set_sensitive(next_button, v);
	gtk_widget_set_sensitive(refreshall_button, v);

	breakpointsEnablePage(v);
	watchpointsEnablePage(v);
	memoryEnablePage(v);
	structuresEnablePage(v);
	configEnablePage(v);
}
