//	bfe2 - cpu state
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
s_cpu cpu;
uint32 i_virtual_segment, i_virtual_offset, i_physical_address, i_size;
char i_description[STRING_LENGTH];
int p_step;
uint32 stack_above, stack_below;

// local
GtkWidget *r_eax_text, *r_ebx_text, *r_ecx_text, *r_edx_text;
GtkWidget *r_ebp_text, *r_esi_text, *r_edi_text, *r_esp_text;
GtkWidget *r_eflags_text, *r_eip_text;
GtkWidget *r_cs_text, *r_ss_text, *r_ds_text, *r_es_text, *r_fs_text, *r_gs_text;
GtkWidget *r_ldtr_text, *r_tr_text, *r_gdtr_text, *r_idtr_text;
GtkWidget *r_dr0_text, *r_dr1_text, *r_dr2_text, *r_dr3_text;
GtkWidget *r_dr4_text, *r_dr5_text, *r_dr6_text, *r_dr7_text;
GtkWidget *r_tr3_text, *r_tr4_text, *r_tr5_text, *r_tr6_text, *r_tr7_text;
GtkWidget *r_cr0_text, *r_cr1_text, *r_cr2_text, *r_cr3_text, *r_cr4_text;
GtkCList *stack_list, *eflags_list;

static gchar *column_names[] = { "", "" };

//////////////////////////////////////////////////////////////////////////

static GtkWidget *new_vbox_1( GtkWidget *box ){
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, FALSE, 0);
	gtk_widget_show(vbox);

	return(vbox);
}


static GtkWidget *new_text_entry( GtkWidget *box, char *name, int width ){
	GtkWidget *hbox, *label, *text;

	hbox = new_hbox(box, FALSE);

	label = gtk_label_new(name);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	text = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(text), FALSE);
	gtk_widget_set_usize(text, width, 20);
	gtk_box_pack_start(GTK_BOX(hbox), text, FALSE, FALSE, 0);
	gtk_widget_show(text);

	return(text);
}


static GtkCList *new_list( GtkWidget *box, int width, int height ){
	GtkWidget *scrolled_window;
	GtkCList *list;

	// create scrolled window
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_usize(scrolled_window, width, height);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(box), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	// create list
	list = (GtkCList *)gtk_clist_new_with_titles(2, column_names);
	gtk_clist_set_column_auto_resize(GTK_CLIST(list), 0, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(list), 1, TRUE);
	gtk_clist_set_selection_mode(list, GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(list));
	gtk_widget_show(GTK_WIDGET(list));

	gtk_clist_column_titles_hide(list);

	return(list);
}


void add_flag( int value, char *name ){
	char temp[2], *row[2];

	snprintf(temp, 2, "%d", value);
	row[0] = temp;
	row[1] = name;
	gtk_clist_append(eflags_list, row);
}

//////////////////////////////////////////////////////////////////////////


void stateInit( GtkWidget *hbox ){
	GtkWidget *vbox, *label;

	vbox = new_vbox_1(hbox);

	label = gtk_label_new("Stack");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	stack_list = new_list(vbox, 140, 140);

	r_dr0_text = new_text_entry(vbox, "DR0", 85);
	r_dr1_text = new_text_entry(vbox, "DR1", 85);
	r_dr2_text = new_text_entry(vbox, "DR2", 85);
	r_dr3_text = new_text_entry(vbox, "DR3", 85);
//	r_dr4_text = new_text_entry(vbox, "DR4", 85);
//	r_dr5_text = new_text_entry(vbox, "DR5", 85);
	r_dr6_text = new_text_entry(vbox, "DR6", 85);
	r_dr7_text = new_text_entry(vbox, "DR7", 85);

	/////////////////////////////////////////////

	vbox = new_vbox_1(hbox);

	r_eax_text = new_text_entry(vbox, "EAX", 85);
	r_ebx_text = new_text_entry(vbox, "EBX", 85);
	r_ecx_text = new_text_entry(vbox, "ECX", 85);
	r_edx_text = new_text_entry(vbox, "EDX", 85);
	r_ebp_text = new_text_entry(vbox, "EBP", 85);
	r_esi_text = new_text_entry(vbox, "ESI", 85);
	r_edi_text = new_text_entry(vbox, "EDI", 85);
	r_esp_text = new_text_entry(vbox, "ESP", 85);

	new_gap(vbox);

	r_tr3_text = new_text_entry(vbox, "TR3", 85);
	r_tr4_text = new_text_entry(vbox, "TR4", 85);
	r_tr5_text = new_text_entry(vbox, "TR5", 85);
	r_tr6_text = new_text_entry(vbox, "TR6", 85);
	r_tr7_text = new_text_entry(vbox, "TR7", 85);

	/////////////////////////////////////////////

	vbox = new_vbox_1(hbox);

	r_eflags_text = new_text_entry(vbox, "EFLAGS", 85);
	r_eip_text = new_text_entry(vbox, "EIP", 85);
	r_cs_text = new_text_entry(vbox, "CS", 110);
	r_ss_text = new_text_entry(vbox, "SS", 110);
	r_ds_text = new_text_entry(vbox, "DS", 110);
	r_es_text = new_text_entry(vbox, "ES", 110);
	r_fs_text = new_text_entry(vbox, "FS", 110);
	r_gs_text = new_text_entry(vbox, "GS", 110);

	new_gap(vbox);

	r_cr0_text = new_text_entry(vbox, "CR0", 85);
	r_cr1_text = new_text_entry(vbox, "CR1", 85);
	r_cr2_text = new_text_entry(vbox, "CR2", 85);
	r_cr3_text = new_text_entry(vbox, "CR3", 85);
	r_cr4_text = new_text_entry(vbox, "CR4", 85);

	/////////////////////////////////////////////

	vbox = new_vbox_1(hbox);

	r_ldtr_text = new_text_entry(vbox, "LDTR", 110);
	r_tr_text = new_text_entry(vbox, "TR", 110);
	r_gdtr_text = new_text_entry(vbox, "GDTR", 170);
	r_idtr_text = new_text_entry(vbox, "IDTR", 170);

	eflags_list = new_list(vbox, 170, 210);
}


void stateUpdate( ){
	char temp[50];

	if(bochs_running) return;

	snprintf(temp, 50, "0x%X", cpu.eax);
	gtk_entry_set_text(GTK_ENTRY(r_eax_text), temp);
	snprintf(temp, 50, "0x%X", cpu.ebx);
	gtk_entry_set_text(GTK_ENTRY(r_ebx_text), temp);
	snprintf(temp, 50, "0x%X", cpu.ecx);
	gtk_entry_set_text(GTK_ENTRY(r_ecx_text), temp);
	snprintf(temp, 50, "0x%X", cpu.edx);
	gtk_entry_set_text(GTK_ENTRY(r_edx_text), temp);
	snprintf(temp, 50, "0x%X", cpu.ebp);
	gtk_entry_set_text(GTK_ENTRY(r_ebp_text), temp);
	snprintf(temp, 50, "0x%X", cpu.esi);
	gtk_entry_set_text(GTK_ENTRY(r_esi_text), temp);
	snprintf(temp, 50, "0x%X", cpu.edi);
	gtk_entry_set_text(GTK_ENTRY(r_edi_text), temp);
	snprintf(temp, 50, "0x%X", cpu.esp);
	gtk_entry_set_text(GTK_ENTRY(r_esp_text), temp);

	snprintf(temp, 50, "0x%X", cpu.eflags);
	gtk_entry_set_text(GTK_ENTRY(r_eflags_text), temp);
	snprintf(temp, 50, "0x%X", cpu.eip);
	gtk_entry_set_text(GTK_ENTRY(r_eip_text), temp);

	snprintf(temp, 50, "0x%X   %s", cpu.cs.segment, (cpu.cs.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_cs_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.ss.segment, (cpu.ss.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_ss_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.ds.segment, (cpu.ds.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_ds_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.es.segment, (cpu.es.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_es_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.fs.segment, (cpu.fs.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_fs_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.gs.segment, (cpu.gs.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_gs_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.ldtr.segment, (cpu.ldtr.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_ldtr_text), temp);
	snprintf(temp, 50, "0x%X   %s", cpu.tr.segment, (cpu.tr.valid ? "valid" : "invalid"));
	gtk_entry_set_text(GTK_ENTRY(r_tr_text), temp);

	snprintf(temp, 50, "B=0x%X  L=0x%X", cpu.gdtr.base, cpu.gdtr.limit);
	gtk_entry_set_text(GTK_ENTRY(r_gdtr_text), temp);
	snprintf(temp, 50, "B=0x%X  L=0x%X", cpu.idtr.base, cpu.idtr.limit);
	gtk_entry_set_text(GTK_ENTRY(r_idtr_text), temp);

	snprintf(temp, 50, "0x%X", cpu.dr0);
	gtk_entry_set_text(GTK_ENTRY(r_dr0_text), temp);
	snprintf(temp, 50, "0x%X", cpu.dr1);
	gtk_entry_set_text(GTK_ENTRY(r_dr1_text), temp);
	snprintf(temp, 50, "0x%X", cpu.dr2);
	gtk_entry_set_text(GTK_ENTRY(r_dr2_text), temp);
	snprintf(temp, 50, "0x%X", cpu.dr3);
	gtk_entry_set_text(GTK_ENTRY(r_dr3_text), temp);
//	snprintf(temp, 50, "0x%X", cpu.dr4);
//	gtk_entry_set_text(GTK_ENTRY(r_dr4_text), temp);
//	snprintf(temp, 50, "0x%X", cpu.dr5);
//	gtk_entry_set_text(GTK_ENTRY(r_dr5_text), temp);
	snprintf(temp, 50, "0x%X", cpu.dr6);
	gtk_entry_set_text(GTK_ENTRY(r_dr6_text), temp);
	snprintf(temp, 50, "0x%X", cpu.dr7);
	gtk_entry_set_text(GTK_ENTRY(r_dr7_text), temp);

	snprintf(temp, 50, "0x%X", cpu.tr3);
	gtk_entry_set_text(GTK_ENTRY(r_tr3_text), temp);
	snprintf(temp, 50, "0x%X", cpu.tr4);
	gtk_entry_set_text(GTK_ENTRY(r_tr4_text), temp);
	snprintf(temp, 50, "0x%X", cpu.tr5);
	gtk_entry_set_text(GTK_ENTRY(r_tr5_text), temp);
	snprintf(temp, 50, "0x%X", cpu.tr6);
	gtk_entry_set_text(GTK_ENTRY(r_tr6_text), temp);
	snprintf(temp, 50, "0x%X", cpu.tr7);
	gtk_entry_set_text(GTK_ENTRY(r_tr7_text), temp);

	snprintf(temp, 50, "0x%X", cpu.cr0);
	gtk_entry_set_text(GTK_ENTRY(r_cr0_text), temp);
	snprintf(temp, 50, "0x%X", cpu.cr1);
	gtk_entry_set_text(GTK_ENTRY(r_cr1_text), temp);
	snprintf(temp, 50, "0x%X", cpu.cr2);
	gtk_entry_set_text(GTK_ENTRY(r_cr2_text), temp);
	snprintf(temp, 50, "0x%X", cpu.cr3);
	gtk_entry_set_text(GTK_ENTRY(r_cr3_text), temp);
	snprintf(temp, 50, "0x%X", cpu.cr4);
	gtk_entry_set_text(GTK_ENTRY(r_cr4_text), temp);

	gtk_clist_freeze(eflags_list);
	gtk_clist_clear(eflags_list);
	add_flag(cpu.eflags&1, "Carry");
	add_flag((cpu.eflags&2)>>1, "-");			// 1
	add_flag((cpu.eflags&4)>>2, "Parity");
	add_flag((cpu.eflags&8)>>3, "-");			// 0
	add_flag((cpu.eflags&16)>>4, "Auxillary");
	add_flag((cpu.eflags&32)>>5, "-");			// 0
	add_flag((cpu.eflags&64)>>6, "Zero");
	add_flag((cpu.eflags&128)>>7, "Sign");
	add_flag((cpu.eflags&256)>>8, "Trap");
	add_flag((cpu.eflags&512)>>9, "Interrupt");
	add_flag((cpu.eflags&1024)>>10, "Direction");
	add_flag((cpu.eflags&2048)>>11, "Overflow");
	add_flag((cpu.eflags&12288)>>12, "IOPL");
	add_flag((cpu.eflags&16384)>>14, "Nested Task");
	add_flag((cpu.eflags&32768)>>15, "-");			// 0
	add_flag((cpu.eflags&65535)>>16, "Resume");
	add_flag((cpu.eflags&131072)>>17, "V86");
	add_flag((cpu.eflags&262144)>>18, "Alignment Check");
	add_flag((cpu.eflags&524288)>>19, "Virtual Interrupt");
	add_flag((cpu.eflags&1048576)>>20, "Virtual Interrupt Pending");
	add_flag((cpu.eflags&2097152)>>21, "ID");
	gtk_clist_thaw(eflags_list);

	stateUpdateStack();
}


void stateUpdateStack( ){
/*
	uint32 stack_value, stack_size, stack_bottom, stack_base;
	uint32 shift, ptr, tos_line;
	char temp[30];
	

	if(!cpu.ss.valid) return;

	tos_line = 0;

	fl_freeze_form(window);
	fl_clear_browser(s_list);

	if((cpu.cr0 & 1) && (cpu.ss.descriptor[1] & 0x400000)){		// check for a 32bit stack
		stack_size = 4;
		shift = 2;
		if(!get_physical((long)&cpu.ss, 0, &stack_base)){
			g_print("BFE: failed to calculate stack base, stack not updated.\n");
			return;
		}
	} else {
		stack_size = 2;
		shift = 1;
		stack_base = cpu.ss.segment<<4;
	}
	stack_bottom = (int)(cpu.esp-(stack_below<<shift)) > 0 ? (cpu.esp-(stack_below<<shift)) : 0;


	for(ptr=cpu.esp+(stack_above<<shift); ptr >= stack_bottom; ptr-=stack_size){
		if(stack_size == 2){
			fprintf(writepipe, "x /1hx 0x%X\n", stack_base+ptr);
		} else {
			fprintf(writepipe, "x /1wx 0x%X\n", stack_base+ptr);
		}
		if(fscanf(readpipe, "%*s %*s 0x%x", &stack_value) == 1){
			if(ptr == cpu.esp){
				snprintf(temp, 30, "TOS\t0x%X", stack_value);
				tos_line = fl_get_browser_maxline(s_list) + 1;
			} else {
				snprintf(temp, 30, "%+d\t0x%X", (int)(ptr-cpu.esp), stack_value);
			}
			if(((stack_size == 2) && !(ptr >= 0x10000)) || (stack_size == 4)){
				if(tos_line == (fl_get_browser_maxline(s_list) + 1)){
					fl_addto_browser(s_list, temp);
				} else fl_add_browser_line(s_list, temp);
			} else tos_line = 0;
		}
		prompt_read();
		if(ptr == 0) break;
	}
	if(tos_line) fl_select_browser_line(s_list, tos_line);
	fl_unfreeze_form(window);
*/
}


int read_cpu ( ){
	if(bochs_offline) return(0);

	return(fscanf(readpipe,
		" eax:%x\n"
		"ebx:%x\n"
		"ecx:%x\n"
		"edx:%x\n"
		"ebp:%x\n"
		"esi:%x\n"
		"edi:%x\n"
		"esp:%x\n"
		"eflags:%x\n"
		"eip:%x\n"
		"cs:s=%x, dl=%x, dh=%x, valid=%u\n"
		"ss:s=%x, dl=%x, dh=%x, valid=%u\n"
		"ds:s=%x, dl=%x, dh=%x, valid=%u\n"
		"es:s=%x, dl=%x, dh=%x, valid=%u\n"
		"fs:s=%x, dl=%x, dh=%x, valid=%u\n"
		"gs:s=%x, dl=%x, dh=%x, valid=%u\n"
		"ldtr:s=%x, dl=%x, dh=%x, valid=%u\n"
		"tr:s=%x, dl=%x, dh=%x, valid=%u\n"
		"gdtr:base=%x, limit=%x\n"
		"idtr:base=%x, limit=%x\n"
		"dr0:%x\n"
		"dr1:%x\n"
		"dr2:%x\n"
		"dr3:%x\n"
//		"dr4:%x\n"
//		"dr5:%x\n"
		"dr6:%x\n"
		"dr7:%x\n"
		"tr3:%x\n"
		"tr4:%x\n"
		"tr5:%x\n"
		"tr6:%x\n"
		"tr7:%x\n"
		"cr0:%x\n"
		"cr1:%x\n"
		"cr2:%x\n"
		"cr3:%x\n"
		"cr4:%x\n"
		"inhibit_mask:%u\n"
		"done\n",
		&cpu.eax,
		&cpu.ebx,
		&cpu.ecx,
		&cpu.edx,
		&cpu.ebp,
		&cpu.esi,
		&cpu.edi,
		&cpu.esp,
		&cpu.eflags,
		&cpu.eip,
		&cpu.cs.segment,		
		&cpu.cs.descriptor[0],
		&cpu.cs.descriptor[1],
		&cpu.cs.valid,
		&cpu.ss.segment,
		&cpu.ss.descriptor[0],
		&cpu.ss.descriptor[1],
		&cpu.ss.valid,
		&cpu.ds.segment,
		&cpu.ds.descriptor[0],
		&cpu.ds.descriptor[1],
		&cpu.ds.valid,
		&cpu.es.segment,
		&cpu.es.descriptor[0],
		&cpu.es.descriptor[1],
		&cpu.es.valid,
		&cpu.fs.segment,
		&cpu.fs.descriptor[0],
		&cpu.fs.descriptor[1],
		&cpu.fs.valid,
		&cpu.gs.segment,
		&cpu.gs.descriptor[0],
		&cpu.gs.descriptor[1],
		&cpu.gs.valid,
		&cpu.ldtr.segment,
		&cpu.ldtr.descriptor[0],
		&cpu.ldtr.descriptor[1],
		&cpu.ldtr.valid,
		&cpu.tr.segment,
		&cpu.tr.descriptor[0],
		&cpu.tr.descriptor[1],
		&cpu.tr.valid,
		&cpu.gdtr.base,
		&cpu.gdtr.limit,
		&cpu.idtr.base,
		&cpu.idtr.limit,
		&cpu.dr0,
		&cpu.dr1,
		&cpu.dr2,
		&cpu.dr3,
//		&cpu.dr4,
//		&cpu.dr5,
		&cpu.dr6,
		&cpu.dr7,
		&cpu.tr3,
		&cpu.tr4,
		&cpu.tr5,
		&cpu.tr6,
		&cpu.tr7,
		&cpu.cr0,
		&cpu.cr1,
		&cpu.cr2,
		&cpu.cr3,
		&cpu.cr4,
		&cpu.inhibit) == (sizeof(s_cpu) / sizeof(int)) );
}
