//	bfe2 - memory watchpoints
//	Copyright (c) 1999-2003 Brand Huntsman and Lee Salzman
//


#include "common.h"
#include "functions.h"


//////////////////////////////////////////////////////////////////////////

// global
s_watchpoint watchpoints[MAX_WATCHPOINTS];

// local
GtkWidget *wp_add_button, *wp_remove_button, *wp_removeall_button, *wp_address_text;
GtkCList *wp_list;
int selected_wp, wp_type;

static gchar *column_names[] = { "", "", "" };

//////////////////////////////////////////////////////////////////////////


int read_watchpoints( ){
	int x, inputChar;
	char type[6];

	if(bochs_offline) return(0);

	for(x = 0; x < MAX_WATCHPOINTS; x++){
		// remove end of line
		inputChar = fgetc(readpipe);
		for(; (inputChar != EOF) && (inputChar != 'w') && (inputChar != 'r') && (inputChar != '<'); inputChar = fgetc(readpipe) );
		if(inputChar == EOF) return(x);
		if((ungetc(inputChar, readpipe) == EOF) || (inputChar == '<')) return(x);

		if(fscanf(readpipe, "%s %x", type, &watchpoints[x].offset) != 2) return(x);
		if(!strcmp(type, "read")) watchpoints[x].type = WP_READ;
		else if(!strcmp(type, "write")) watchpoints[x].type = WP_WRITE;
		else return(x);
		watchpoints[x].enabled = 1;
	}
	return(x);
}


void set_watchpoint_type( GtkWidget *widget, gpointer data ){
	switch((int)data){
	case WP_READ:
		wp_type = WP_READ;
		break;
	case WP_WRITE:
		wp_type = WP_WRITE;
		break;
	}
}


void add_watchpoint( GtkWidget *widget, gpointer data ){
	const char *address;

	address = gtk_entry_get_text(GTK_ENTRY(wp_address_text));

	switch(wp_type){
	case WP_READ:
		fprintf(writepipe, "watch read %s\n", address);
		break;
	case WP_WRITE:
		fprintf(writepipe, "watch write %s\n", address);
		break;
	}
	prompt_read();
	if(!bochs_restarting) watchpointsUpdate(0);
	gtk_entry_set_text(GTK_ENTRY(wp_address_text), "");
}


void remove_watchpoint( GtkWidget *widget, gpointer data ){
	if(selected_wp == -1) return;

	switch(watchpoints[selected_wp].type){
	case WP_READ:
g_print("unwatch read 0x%x\n", watchpoints[selected_wp].offset);
		fprintf(writepipe, "unwatch read 0x%x\n", watchpoints[selected_wp].offset);
		break;
	case WP_WRITE:
g_print("unwatch write 0x%x\n", watchpoints[selected_wp].offset);
		fprintf(writepipe, "unwatch write 0x%x\n", watchpoints[selected_wp].offset);
		break;
	}
	prompt_read();
	watchpointsUpdate(0);
}


void removeall_watchpoints( GtkWidget *widget, gpointer data ){
	fprintf(writepipe, "unwatch\n");
	prompt_read();
	watchpointsUpdate(0);
}


void wp_selected( GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data ){
	selected_wp = row;
}


void wp_unselected( GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data ){
	selected_wp = -1;
}


//////////////////////////////////////////////////////////////////////////


void watchpointsInit( GtkWidget *vbox ){
	GtkWidget *hbox, *vbox2, *scrolled_window, *button, *label;

	hbox = new_hbox(vbox, TRUE);

	// create scrolled window
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled_window, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_window);

	// create list
	wp_list = (GtkCList *)gtk_clist_new_with_titles(3, column_names);
	gtk_clist_set_column_auto_resize(GTK_CLIST(wp_list), 0, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(wp_list), 1, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(wp_list), 2, TRUE);
	gtk_clist_set_selection_mode(wp_list, GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(wp_list));
	gtk_widget_show(GTK_WIDGET(wp_list));

	gtk_clist_column_titles_hide(wp_list);

	// setup selection handler
	gtk_clist_set_selection_mode(wp_list, GTK_SELECTION_SINGLE);
	gtk_signal_connect(GTK_OBJECT(wp_list), "select_row", GTK_SIGNAL_FUNC(wp_selected), NULL);
	gtk_signal_connect(GTK_OBJECT(wp_list), "unselect_row", GTK_SIGNAL_FUNC(wp_unselected), NULL);

	vbox2 = new_vbox(hbox, FALSE);
	hbox = new_hbox(vbox2, FALSE);

	// add button
	wp_add_button = gtk_button_new_with_label(" Add ");
	gtk_signal_connect(GTK_OBJECT(wp_add_button), "clicked", GTK_SIGNAL_FUNC(add_watchpoint), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), wp_add_button, TRUE, TRUE, 0);
	gtk_widget_show(wp_add_button);

	// remove button
	wp_remove_button = gtk_button_new_with_label(" Remove ");
	gtk_signal_connect(GTK_OBJECT(wp_remove_button), "clicked", GTK_SIGNAL_FUNC(remove_watchpoint), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), wp_remove_button, FALSE, FALSE, 0);
	gtk_widget_show(wp_remove_button);

	// remove all button
	wp_removeall_button = gtk_button_new_with_label(" Remove All ");
	gtk_signal_connect(GTK_OBJECT(wp_removeall_button), "clicked", GTK_SIGNAL_FUNC(removeall_watchpoints), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), wp_removeall_button, FALSE, FALSE, 0);
	gtk_widget_show(wp_removeall_button);

	hbox = new_hbox(vbox2, FALSE);

	// read radio button
	button = gtk_radio_button_new_with_label(NULL, " Read ");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(set_watchpoint_type), (gpointer)WP_READ);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	// write radio button
	button = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(button)), " Write ");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(set_watchpoint_type), (gpointer)WP_WRITE);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_widget_show(button);

	// address text field
	wp_address_text = gtk_entry_new();
	gtk_widget_set_usize(wp_address_text, 145, 0);
	gtk_signal_connect(GTK_OBJECT(wp_address_text), "activate", GTK_SIGNAL_FUNC(add_watchpoint), NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), wp_address_text, FALSE, FALSE, 0);
	gtk_widget_show(wp_address_text);

	// message label
	label = gtk_label_new("Bochs 1.4 and below does not support remove, it might be supported in newer versions.");
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	selected_wp = 0;
	wp_type = WP_READ;
}


void watchpointsUpdate( int redraw ){
	int x, length;
	char address[20], *row[3];
	const char *r_type = "Read", *w_type = "Write";

	if(bochs_offline) return;

	if(!bochs_restarting && !redraw){
		for(x = 0; x < MAX_WATCHPOINTS; x++) watchpoints[x].enabled = 0;
		fprintf(writepipe, "watch\n");
		read_watchpoints();
		prompt_read();
	}

	gtk_clist_clear(wp_list);

	row[1] = address;
	row[2] = NULL;
	length = 0;

	for(x = 0; (x < MAX_WATCHPOINTS) && watchpoints[x].enabled; x++){
		switch(watchpoints[x].type){
		case WP_READ:
			row[0] = (char *)r_type;
			snprintf(address, 20, "0x%.8X", watchpoints[x].offset);
			break;
		case WP_WRITE:
			row[0] = (char *)w_type;
			snprintf(address, 20, "0x%.8X", watchpoints[x].offset);
			break;
		}
		gtk_clist_append(wp_list, row);
		length++;

		if(bochs_restarting){
//			wp_type = watchpoints[x].type;
//			gtk_entry_set_text(GTK_ENTRY(wp_address_text), address);
//			add_watchpoint(NULL, 0);

  // TODO: need to restore wp_type or else the radio buttons won't match

		}
	}
	if(watchpoints[0].enabled && !redraw) gtk_clist_select_row(wp_list, length-1, 0);
}


void watchpointsEnablePage( gboolean v ){
	gtk_widget_set_sensitive(wp_add_button, v);
	gtk_widget_set_sensitive(wp_remove_button, v);
	gtk_widget_set_sensitive(wp_removeall_button, v);
	gtk_widget_set_sensitive(wp_address_text, v);
}


void watchpointsSelect( int wp ){
	gtk_clist_set_text(wp_list, wp, 2, "<--");
	gtk_clist_select_row(wp_list, wp, 0);
}
